// Example parameters to register and query the Goat milk price quote service.
// Note that register-org gets the tmodel-key from the output of register-tmodel.
//
// Look up ntis-gov:naics taxonomy classifications at:
// http://www.census.gov/epcd/naics02/naicod02.htm
//
// You can verify that your data was entered by looking it up in IBM's Test
// UDDI Registry: https://uddi.ibm.com/testregistry/registry.html (use the 
// same user name and password as below).
//
// register-tmodel ssanner@gmail.com mie354test
// register-org ssanner@gmail.com mie354test UUID:70539A30-563A-11DA-8A45-000629DC0A53
// query ssanner@gmail.com mie354test "Goat farming" 112420 

package estore.ui.webservice;

import java.util.*;
import java.net.PasswordAuthentication;
import javax.xml.registry.*;
import javax.xml.registry.infomodel.*;

public class UDDIInterface {
	
	  public static BusinessQueryManager     businessQueryManager;
	  public static BusinessLifeCycleManager businessLifeCycleManager;
	  
	  // Instantiates connections to the above managers
	  public void connectToRegistry(String user, String pass) {

	      try {

	          Properties prop = new Properties();
	          prop.setProperty("javax.xml.registry.queryManagerURL",
	              "http://uddi.ibm.com/testregistry/inquiryapi");
	          prop.setProperty("javax.xml.registry.lifeCycleManagerURL",
	              "https://uddi.ibm.com/testregistry/publishapi");
	          prop.setProperty("javax.xml.registry.factoryClass",
	              "com.sun.xml.registry.uddi.ConnectionFactoryImpl");

	          //Provide your UDDI account user name and password.
	          //You must create a real UDDI account for this work.
	          PasswordAuthentication passwdAuth =
	              new PasswordAuthentication(user, pass.toCharArray());
	          Set creds = new HashSet();
	          creds.add(passwdAuth);

	          ConnectionFactory connFac = ConnectionFactory.newInstance();
	          connFac.setProperties(prop);
	          Connection connection = connFac.createConnection();

	          connection.setCredentials(creds);

	          RegistryService registryService = connection.getRegistryService();
	          businessQueryManager = registryService.getBusinessQueryManager();
	          businessLifeCycleManager = registryService.getBusinessLifeCycleManager();

	          if (businessQueryManager == null || businessLifeCycleManager == null) {
	        	  System.out.println("Could not connect...exiting.");
	        	  System.exit(1);
	          }
	      }
	     catch (JAXRException e) {
	         e.printStackTrace(System.out);
	     }
	  }
	
	// TODO: Modify code for the service tModel you want to register.	  
	public void addTModel() {
	    try {

	        // 1. Create a new Concept.
	        // The null parameter signifies that this concept has no parent.
	        Concept wsdlConcept =
	            businessLifeCycleManager.createConcept(
	                 null,
	                "Goat milk price quotes service",
	                 null);
	        InternationalString description =
	            businessLifeCycleManager.createInternationalString(
	                "A service that provides current " +
	                "buying and selling prices for goat's milk.");
	        wsdlConcept.setDescription(description);


	        // 2. Create wsdlSpec classification. 
	        // The type (scheme) for that classification is uddi-org:types.
	        ClassificationScheme sc1 = findClassificationScheme("uddi-org:types");
	        Classification wsdlSpecClassification =
	            businessLifeCycleManager.createClassification(
	                sc1, "wsdlSpec", "wsdlSpec");

	        // 3. Create industry classification using ntis-gov:naics scheme.
	        // http://www.census.gov/epcd/naics02/naicod02.htm
	        ClassificationScheme sc2 = findClassificationScheme("ntis-gov:naics");
	        Classification industryClassification =
	            businessLifeCycleManager.createClassification(
	                sc2, "Goat farming", "112420");


	        //4. Create an external link for the WSDL document location. 
	        //NOTE: This must point to a valid URL, or JAXR will throw an exception.
	        String wsdlUrl = "http://goatmilk.com/getprices?q=WSDL";
	        String wsdlDesc = "WSDL service for goat milk price quotes";
	        ExternalLink wsdlLink =
	            businessLifeCycleManager.createExternalLink(wsdlUrl, wsdlDesc);


	        //5. Add the two classifications to the Concept.
	        Collection classifications = new ArrayList();
	        classifications.add(wsdlSpecClassification);
	        classifications.add(industryClassification);
	        wsdlConcept.addClassifications(classifications);


	        //6. Add the external link to the Concept.
	        wsdlConcept.addExternalLink(wsdlLink);


	        //7. Save the new concept in UDDI.
	        Collection newConcepts = new ArrayList();
	        newConcepts.add(wsdlConcept);
	        BulkResponse resp = businessLifeCycleManager.saveConcepts(newConcepts);

	        //8. Check if we've succeded.
	        if (resp.getExceptions() == null) {
	            //Success.
	             System.out.println("\ntModel saved");
	             Collection keys = resp.getCollection();
	             Iterator it = keys.iterator();
	             while (it.hasNext()) {
	                 Key o = (Key) it.next();
	                 String id = o.getId();
	                 System.out.println("TModel key is: " + id);
	            }
		        
	        } else {
	            //Something went wrong.
	            for (Iterator iterator = resp.getExceptions().iterator(); iterator.hasNext();) {
	                RegistryException registryException = 
	                    (RegistryException) iterator.next();
	                System.out.println(registryException.toString());
	            }
	        }

		} catch (Exception e) {
		     e.printStackTrace(System.out);
		}
	}

	// TODO: Modify code for the organization and service you want to register.	  
	public void addOrganization(String tmodel_key) {

	    try {

	        //1. Create an organization
	        Organization org =
	            businessLifeCycleManager.createOrganization("Goat Milk, Inc.");
	        InternationalString str =
	            businessLifeCycleManager.createInternationalString(
	                "A goat milk price quoting service");
	        org.setDescription(str);

	         //Create primary contact, other organization info 
	        User primaryContact = businessLifeCycleManager.createUser();
	        PersonName pName = 
	            businessLifeCycleManager.createPersonName("Scott Sanner");
	        primaryContact.setPersonName(pName);
	        TelephoneNumber tNum = 
	            businessLifeCycleManager.createTelephoneNumber();
	        tNum.setNumber("800-555-5555");
	        Collection phoneNumbers = new ArrayList();
	        phoneNumbers.add(tNum);
	        primaryContact.setTelephoneNumbers(phoneNumbers);
	        EmailAddress emailAddress =
	            businessLifeCycleManager.createEmailAddress("email@goatmilk.com");
	        Collection emailAddresses = new ArrayList();
	        emailAddresses.add(emailAddress);
	        primaryContact.setEmailAddresses(emailAddresses);
	        org.setPrimaryContact(primaryContact);

	        //Classify your organization
	        //Might also add geographic or other classifications
	        ClassificationScheme sc = findClassificationScheme("ntis-gov:naics");
	        Classification classification = businessLifeCycleManager.createClassification(
	        		sc, "Goat farming", "112420");
	        Collection classifications = new ArrayList();
	        classifications.add(classification);
	        org.addClassifications(classifications);

	        //2. Create a new service
	        Service cruiseImpl = 
	            businessLifeCycleManager.createService("Goat milk service");

	        //3. Create a service binding, specify service access URI
	        ServiceBinding binding = businessLifeCycleManager.createServiceBinding();
	        //Substitute your own access URI where your service can be invoked
	        binding.setAccessURI("http://goatmilk.com/getprices");

	        //4. Associate CruiseService tModel with service binding
	        //(a) Retrieve tModel associated with key
	        //Key for CruiseService tModel
	        //You obtained that when searching for industry-specific tModels
	        RegistryObject rob = 
	            businessQueryManager.getRegistryObject(
	                tmodel_key, 
	                BusinessLifeCycleManager.CONCEPT);
	        if (rob == null) {
	             throw new NullPointerException("Registry object not found");
	        }

	        //(b) Create new specification link, associate it with tModel 
	        SpecificationLink specLink = 
	            businessLifeCycleManager.createSpecificationLink();
	        specLink.setSpecificationObject(rob);

	        //(c) Add specification link to service binding
	        binding.addSpecificationLink(specLink);


	        //Add service binding to service
	        cruiseImpl.addServiceBinding(binding);

	         //Add service to organization
	        org.addService(cruiseImpl);

	        //Save organization 
	        Collection orgs = new ArrayList();
	        orgs.add(org);
	        BulkResponse response;
	        try {
	            response = businessLifeCycleManager.saveOrganizations(orgs);
	        } catch (Exception e) {
	            e.printStackTrace();
	            return;
	        }

	        //Check if everything went well, print new organization key
	        Collection exceptions = response.getExceptions();
	        if (exceptions == null) {
	             System.out.println("\nOrganization saved");
	             Collection keys = response.getCollection();
	             Iterator it = keys.iterator();
	             while (it.hasNext()) {
	                 Key o = (Key) it.next();
	                 String id = o.getId();
	                 System.out.println("Organization key is: " + id);
	                }
	            }
	            else {
	            //Something went wrong
	                System.out.println("Couldn't save organization.");
	                for (Iterator iterator = exceptions.iterator(); iterator.hasNext();) {
	                    JAXRException e = (JAXRException) iterator.next();
	                    System.out.println(e.getMessage());
	                }
	            }
	    }
	    catch (JAXRException e) {
	        e.printStackTrace();
	    }
	}

	// No modification needed, but take a quick look at the code.
	public void query(String type_desc, String type_id) {

	    try {

	        //1. Specify search criteria
	        //(a) Search-based industry classification
	        ClassificationScheme naicsScheme = 
	        	findClassificationScheme("ntis-gov:naics");
	        Classification typeSpecClassification =
	            businessLifeCycleManager.createClassification(naicsScheme,
	            	type_desc, type_id);

	        //(b) Lookup for wsdlSpec type tModels only
	        ClassificationScheme uddiOrgType = 
	        	findClassificationScheme("uddi-org:types");
	        Classification wsdlSpecClassification =
	                businessLifeCycleManager.createClassification(uddiOrgType,
	                "wsdlSpec", "wsdlSpec");

	        Collection classifications = new ArrayList();
	        classifications.add(typeSpecClassification);
	        classifications.add(wsdlSpecClassification);


	        //2. Invoke findConcepts
	        BulkResponse results = businessQueryManager.findConcepts(
	             null,
	             null,
	             classifications,
	             null,
	             null);


	       //3. Iterate through results, find information about tModels
	       Collection col = results.getCollection();
	       int result_count = 0;
	       if (col != null) {
	           for (Iterator iterator = col.iterator(); iterator.hasNext(); ) {
	               Concept concept = (Concept) iterator.next();

	               //tModel's name and description
	               System.out.println("\n\nResult #"  + ++result_count + ":\n----------");
	               System.out.println("Description: " + concept.getDescription().getValue());
	               System.out.println("Name:        " + concept.getName().getValue());
	               //tModel key
	               System.out.println("tModel Key:  " + concept.getKey().getId());

	               //4. Find external links, retrieve and save WSDL documents
	               Collection links = concept.getExternalLinks();
	               for (Iterator ito = links.iterator(); ito.hasNext();) {
	                   ExternalLink externalLink = (ExternalLink) ito.next();
	                   String externalURI = externalLink.getExternalURI();

	                   //Retrieve the URL's content and save it into a file
	                   System.out.println("External content at: " + externalURI);
	                }
	            }
	       }
	    }
	    catch (JAXRException e) {
	        e.printStackTrace();
	    }
	}

	// Returns the first ClassificationScheme of multiple matching entries.
	public ClassificationScheme findClassificationScheme(String name) {
        ArrayList name_list = new ArrayList(); name_list.add(name);
        try {
        	BulkResponse br = businessQueryManager.findClassificationSchemes(null, 
        			name_list, null, null);
        	return (ClassificationScheme)br.getCollection().iterator().next();
        } catch (JAXRException e) {
        	System.out.println("ERROR: findClassificationScheme: " + e);
        	e.printStackTrace(System.out);
        	System.exit(1);
        }
        return null;
	}
	
	// Helper function
	public Collection makeCollection(String element) {
		ArrayList l = new ArrayList();
		l.add(element);
		return l;
	}
	
	// TODO: Fill in the control structure at the end of this method.
	public static void main(String[] args) {
		if (args.length < 1 ||
			!(args[0].equals("register-tmodel") || 
			  args[0].equals("register-org") || args[0].equals("query")) ||
			(args[0].equals("register-tmodel") && args.length != 3) ||
			(args[0].equals("register-org")    && args.length != 4) ||
			(args[0].equals("query")           && args.length != 5)) {
			System.out.println("\nIncorrect arguments, choose one of following:");
			System.out.println("[1] register-tmodel <user> <pass>");
			System.out.println("[2] register-org <user> <pass> <uuid>");
			System.out.println("[3] query <user> <pass> <type_desc> <type_ID>");
			System.exit(1);
		}
		
		// TODO: Create a new UDDIInterface
		UDDIInterface uddi = null;
		
		// TODO: Connect to registry

		// TODO: Perform command by calling class methods with parameters
	}
	
}
