/***********************************************************************
 * Created on Nov 2, 2005
 *
 * Author: Alexey Tugonov
 * Modified by: Scott Sanner
 ***********************************************************************/
package estore.ui.webservice;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

import org.dom4j.Attribute;
import org.dom4j.Node;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import comshell.*;
import estore.dataman.*;
import estore.shared.*;
import estore.ui.shell.TestInterface;

import java.io.*;
import java.util.*;

/** This class takes XML as input, decodes it, passes it to the
 *  EStoreInterface, encodes the output as XML and returns it.
 */
public class XMLEStoreInterface {
	
	protected EStoreInterface _es;
	
		// Constructor
		public XMLEStoreInterface(EStoreInterface e) 
		{
			_es = e;	
		}
		
		//////////////////////////////////////////////////////////////////////
		// Executes the command described by the XML and encodes the
		// result as XML and returns it.  The XPath queries and return
		// XML document contruction conform to the datatypes defined in 
		// XMLEStoreInterface.xml, which are used in EStoreWebService.wsdl.
		//
		// See the *.soap files in this directory for actual SOAP documents
		// that were passed to EStoreWebService.
		//////////////////////////////////////////////////////////////////////

		public Document doCommand (Document input) 
		{	
			try {
				
				// Obtain the name of the command (the root element)
				String command = GetElementName(input, "/*"); 
				
				/***********************************************************
			    * Command: Add items to shopping cart
			    ***********************************************************/
				if(command.equals("addItem")){
					
					// Obtain the parameters
					String user  = GetElementText(input, "/addItem/user");
					String item  = GetElementText(input, "/addItem/itemName");
					String quant = GetElementText(input, "/addItem/quantity");
					int quantity = Integer.parseInt(quant);
					
					// Call the EStore method
					System.out.println(user + ", " + item + ", " + quantity);
					boolean success = _es.addItem(user, item, quantity);
					
					// Generate the return document
					Document document = DocumentHelper.createDocument();
			        Element body = document.addElement( "result" );
			        Element root = body.addElement( "success" );
			        root.addText( "" + success );
			        return document;				
				}
				
				/***********************************************************
				 * Command: Remove items from shopping cart
				 ***********************************************************/						
				else if (command.equals("removeItem")){
					
					// Obtain the parameters
					String user  = GetElementText(input, "/removeItem/user");
					String item  = GetElementText(input, "/removeItem/itemName");
					String quant = GetElementText(input, "/removeItem/quantity");
					int quantity = Integer.parseInt(quant);
					
					// Call the EStore method
					boolean success = _es.removeItem(user, item, quantity);
					
					// Generate the return document
					Document document = DocumentHelper.createDocument();
					Element body = document.addElement( "result" );
			        Element root = body.addElement( "success" );
			        root.addText( "" + success );
					return document;				
				}
					
				/***********************************************************
			     * Command: Get all items in user's shopping cart
			     ***********************************************************/
				else if (command.equals("listSCItems")){
					
					// Obtain the parameter
					String user  = GetElementText(input, "/listSCItems/user");
					
					// Call the EStore method
					Item[] result = _es.listSCItems(user);
					
			        // Encode array as XML document and return
					Document document = DocumentHelper.createDocument();
			        Element root = document.addElement( "result" );
			        for (int i = 0; i < result.length; i++) {
						Element item = root.addElement( "item" );
						item.addElement( "name" ).addText(  result[i]._sName);
						item.addElement( "PID"  ).addText(  ""+result[i]._nPID);
						item.addElement( "quantity" ).addText( ""+result[i]._nQuantity);
						item.addElement( "price" ).addText( ""+result[i]._bdPrice);
			        }
					return document;
				}
					
				/***********************************************************
			     * Command: Get all items in inventory
			     ***********************************************************/
				else if (command.equals("listInventoryItems")){
					
					// Call the EStore method (no parameters required)
					Item[] result = _es.listInventoryItems();
					
			        // Encode array as XML document and return
					Document document = DocumentHelper.createDocument();
			        Element root = document.addElement( "result" );
			        for (int i = 0; i < result.length; i++) {
						Element item = root.addElement( "item" );
						item.addElement( "name" ).addText(  result[i]._sName);
						item.addElement( "PID"  ).addText(  ""+result[i]._nPID);
						item.addElement( "quantity" ).addText( ""+result[i]._nQuantity);
						item.addElement( "price" ).addText( ""+result[i]._bdPrice);
			        }
					return document;
				}
				
				/***********************************************************
			     * Command: Get info for item
			     ***********************************************************/
				else if (command.equals("getItemInfo")){
					
					// Obtain the parameter
					String item_name = GetElementText(input, "/getItemInfo/itemName");
					
					// Call the EStore method
					Item result = _es.getItemInfo(item_name);

			        // Encode Item as XML document and return
					Document document = DocumentHelper.createDocument();
			        Element root = document.addElement( "result" );
			        Element singleItem = root.addElement( "singleItem" );
			        singleItem.addElement( "name" ).addText(  result._sName);
			        singleItem.addElement( "PID"  ).addText(  ""+result._nPID);
			        singleItem.addElement( "quantity" ).addText( ""+result._nQuantity);
			        singleItem.addElement( "price" ).addText( ""+result._bdPrice);
					return document;
				}
				
				/***********************************************************
			     * Command: Checkout shopping cart for user
			     ***********************************************************/	
				else if (command.equals("checkout")){
					
					// Obtain the parameters
					String user        = GetElementText(input, "/checkout/user");
					String email       = GetElementText(input, "/checkout/email");
					String paymentInfo = GetElementText(input, "/checkout/paymentInfo");
					
					// Call the EStore method
					boolean success = _es.checkout(user, email, paymentInfo);
					
					// Generate the return document
					Document document = DocumentHelper.createDocument();
			        Element body = document.addElement( "result" );
			        Element root = body.addElement( "success" );
			        root.addText( "" + success );
			        return document;				
				}
				
				/***********************************************************
			     * Error
			     ***********************************************************/	
				else {
					// XML error, return empty document
					return DocumentHelper.createDocument();
				}
				
			} catch (Exception e) {
				// Signal error to user and return the empty document
				System.out.println("ERROR: " + e);
				e.printStackTrace(System.out);
				return DocumentHelper.createDocument();
			}
		}
				
		//////////////////////////////////////////////////////////////////////
		// XPath query helper methods.
		//////////////////////////////////////////////////////////////////////

		// Returns the text of the first element selected by the XPath query
		public static String GetElementName(Document d, String xpath)
		{
			return GetElementFeature(d, xpath, true);
	    }
				
		// Returns the text of the first element selected by the XPath query
		public static String GetElementText(Document d, String xpath)
		{
			return GetElementFeature(d, xpath, false);
	    }
		
		// If (retrieve_name == true) returns name of first matching element,
		// otherwise it returns the text of the first matching element
		public static String GetElementFeature(Document d, String xpath, 
											   boolean retrieve_name) 
		{
			// Execute the XPath query
			List l = d.selectNodes(xpath);
			
			// If no nodes match query, return null
			if (l.size()==0) {
				return null;
			} 
			
	    	// Obtain the first element
			Node n = (Node)l.get(0);
			
			// We were expecting an Element (not an Attribute or other Node subtype)
			if ( !(n instanceof Element) ) {
				return null;
			}
			
			// Return the text of this element
			if (retrieve_name) {
				return ((Element)n).getQualifiedName();
			} else {
				return ((Element)n).getText();
			}
	    }
		
		//////////////////////////////////////////////////////////////////////
		// The following methods generate test input documents conforming
		// to the datatypes defined in XMLEStoreInterface.xml which are
		// used in EStoreWebService.wsdl.
		//////////////////////////////////////////////////////////////////////
		
		public static Document testAddItemDocument() {
	        Document document = DocumentHelper.createDocument();
	        Element addItemInput = document.addElement( "addItem" );
	        addItemInput.addElement( "user" ).addText( "James" );
	        addItemInput.addElement( "itemName" ).addText( "Widget1" );
	        addItemInput.addElement( "quantity" ).addText( "10" );
	        
	        return document;
	    }
		
		public static Document testRemoveItemDocument() {
	        Document document = DocumentHelper.createDocument();
	        Element removeItemInput = document.addElement( "removeItem" );
	        removeItemInput.addElement( "user" ).addText( "James" );
	        removeItemInput.addElement( "itemName" ).addText( "Widget1" );
			removeItemInput.addElement( "quantity" ).addText( "5" );
	        
	        return document;
	    }
		
		public static Document testlistSCItemsDocument() {
	        Document document = DocumentHelper.createDocument();
	        Element listSCItemsInput = document.addElement( "listSCItems" );
	        listSCItemsInput.addElement( "user" ).addText( "James" );
	        
	        return document;
	    }
		
		public static Document testlistInventoryItemsDocument() {
	        Document document = DocumentHelper.createDocument();
	        Element listInventory = document.addElement( "listInventoryItems" );
	        
	        return document;
	    }
		
		public static Document testGetItemInfoDocument() {
	        Document document = DocumentHelper.createDocument();
	        Element getItemInfoInput = document.addElement( "getItemInfo");
			getItemInfoInput.addElement( "itemName" ).addText( "Widget1" );
	        
	        return document;
	    }
		
		public static Document testCheckoutDocument() {
	        Document document = DocumentHelper.createDocument();
	        Element checkoutInput = document.addElement( "checkout");
			checkoutInput.addElement( "user" ).addText( "James" );
			checkoutInput.addElement( "email" ).addText( "james@example.com" );
			checkoutInput.addElement( "paymentInfo" ).addText( "Visa 1234 5/6" );
		    return document;
	    }
		
		//////////////////////////////////////////////////////////////////////
		// Main method for testing.
		//////////////////////////////////////////////////////////////////////
		
		public static void main(String [] arges)
		{			
	        XMLEStoreInterface shell = new XMLEStoreInterface(new EStore());

	        // Test each of the six methods
	        for (int count = 1; count <= 6; count++) {
		        Document doc_input = null, doc_output = null;

		        switch (count) {
	        	case 1: doc_input = testAddItemDocument();     break;
	        	case 2: doc_input = testRemoveItemDocument();  break;
	        	case 3: doc_input = testlistSCItemsDocument(); break;
	        	case 4: doc_input = testlistInventoryItemsDocument(); break;
	        	case 5: doc_input = testGetItemInfoDocument(); break;
	        	case 6: doc_input = testCheckoutDocument();    break;
	        	default: break;
	        	}

	        	// Run test and print out input and output
	        	System.out.println("Test: " + count + "\n-------");
	        	System.out.println("Input  XML: " + doc_input.asXML());
			    doc_output = shell.doCommand(doc_input);
			    System.out.println("Output XML: " + doc_output.asXML() + "\n");
	        }
	        
		}
}
